@@ -11,6 +11,7 @@
 #include <errno.h>
 #include <signal.h>
 #include <pthread.h>
+#include <inttypes.h>
 #include "network_ipc.h"
 
 static pthread_t async_thread_id;
@@ -23,7 +24,11 @@ struct async_lib {
 	terminated	end;
 };
 
-static int handle = 0;
+static enum async_thread_state {
+	ASYNC_THREAD_INIT,
+	ASYNC_THREAD_RUNNING,
+	ASYNC_THREAD_DONE
+} running = ASYNC_THREAD_INIT;
 
 static struct async_lib request;
 
@@ -37,14 +42,15 @@ static void *swupdate_async_thread(void *data)
 	sigset_t saved_mask;
 	struct timespec zerotime = {0, 0};
 	struct async_lib *rq = (struct async_lib *)data;
-	int swupdate_result;
+	int swupdate_result = FAILURE;
 
 	sigemptyset(&sigpipe_mask);
 	sigaddset(&sigpipe_mask, SIGPIPE);
 
 	if (pthread_sigmask(SIG_BLOCK, &sigpipe_mask, &saved_mask) == -1) {
-		  perror("pthread_sigmask");
-		    exit(1);
+		perror("pthread_sigmask");
+		swupdate_result = FAILURE;
+		goto out;
 	}
 	/* Start writing the image until end */
 
@@ -53,8 +59,13 @@ static void *swupdate_async_thread(void *data)
 			break;
 
 		rq->wr(&pbuf, &size);
-		if (size)
-			swupdate_image_write(pbuf, size);
+		if (size) {
+			if (swupdate_image_write(pbuf, size) != size) {
+				perror("swupdate_image_write failed");
+				swupdate_result = FAILURE;
+				goto out;
+			}
+		}
 	} while(size > 0);
 
 	ipc_end(rq->connfd);
@@ -65,20 +76,22 @@ static void *swupdate_async_thread(void *data)
 
 	swupdate_result = ipc_wait_for_complete(rq->get);
 
-	handle = 0;
-
 	if (sigtimedwait(&sigpipe_mask, 0, &zerotime) == -1) {
 		// currently ignored
 	}
 
 	if (pthread_sigmask(SIG_SETMASK, &saved_mask, 0) == -1) {
-		  perror("pthread_sigmask");
+		perror("pthread_sigmask");
+		swupdate_result = FAILURE;
+		goto out;
 	}
 
+out:
+	running = ASYNC_THREAD_DONE;
 	if (rq->end)
 		rq->end((RECOVERY_STATUS)swupdate_result);
 
-	pthread_exit(NULL);
+	pthread_exit((void*)(intptr_t)(swupdate_result == SUCCESS));
 }
 
 /*
@@ -86,20 +99,21 @@ static void *swupdate_async_thread(void *data)
  * to let build the ipc library without
  * linking pctl code
  */
-static pthread_t start_ipc_thread(void *(* start_routine) (void *), void *arg)
+static void start_ipc_thread(void *(* start_routine) (void *), void *arg)
 {
 	int ret;
-	pthread_t id;
 	pthread_attr_t attr;
 
 	pthread_attr_init(&attr);
 	pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_JOINABLE);
 
-	ret = pthread_create(&id, &attr, start_routine, arg);
+	ret = pthread_create(&async_thread_id, &attr, start_routine, arg);
 	if (ret) {
-		exit(1);
+		perror("ipc thread creation failed");
+		return;
 	}
-	return id;
+
+	running = ASYNC_THREAD_RUNNING;
 }
 
 /*
@@ -112,8 +126,16 @@ int swupdate_async_start(writedata wr_func, getstatus status_func,
 	struct async_lib *rq;
 	int connfd;
 
-	if (handle)
+	switch (running) {
+	case ASYNC_THREAD_INIT:
+		break;
+	case ASYNC_THREAD_DONE:
+		pthread_join(async_thread_id, NULL);
+		running = ASYNC_THREAD_INIT;
+		break;
+	default:
 		return -EBUSY;
+	}
 
 	rq = get_request();
 
@@ -128,11 +150,9 @@ int swupdate_async_start(writedata wr_func, getstatus status_func,
 
 	rq->connfd = connfd;
 
-	async_thread_id = start_ipc_thread(swupdate_async_thread, rq);
+	start_ipc_thread(swupdate_async_thread, rq);
 
-	handle++;
-
-	return handle;
+	return running != ASYNC_THREAD_INIT;
 }
 
 int swupdate_image_write(char *buf, int size)
