@@ -212,6 +212,8 @@ int ipc_notify_connect(void)
 	if (ret || msg.type != ACK) {
 		fprintf(stdout, "Notify connection handshake failed..\n");
 		close(connfd);
+		if (ret >= 0)
+			ret = -EIO;
 		return ret;
 	}
 
@@ -308,8 +310,18 @@ int ipc_inst_start(void)
  */
 int ipc_send_data(int connfd, char *buf, int size)
 {
-	ssize_t ret = write(connfd, buf, (size_t)size);
-	return ret != size ? -1 : (int)ret;
+	ssize_t ret;
+	ssize_t len = size;
+
+	while (len) {
+		ret = write(connfd, buf, (size_t)size);
+		if (ret < 0)
+			return ret;
+		len -= ret;
+		buf += ret;
+	}
+
+	return size;
 }
 
 void ipc_end(int connfd)
