@@ -385,6 +385,7 @@ static int parse_common_attributes(parsertype p, void *elem, struct img_type *im
 {
 	char seek_str[MAX_SEEK_STRING_SIZE];
 	const char* compressed;
+	unsigned long offset = 0;
 
 	/*
 	 * GET_FIELD_STRING does not touch the passed string if it is not
@@ -401,15 +402,24 @@ static int parse_common_attributes(parsertype p, void *elem, struct img_type *im
 	GET_FIELD_STRING(p, elem, "mtdname", image->mtdname);
 	GET_FIELD_STRING(p, elem, "filesystem", image->filesystem);
 	GET_FIELD_STRING(p, elem, "type", image->type);
+	get_field(p, elem, "offset", &offset);
 	GET_FIELD_STRING(p, elem, "offset", seek_str);
 	GET_FIELD_STRING(p, elem, "data", image->type_data);
 	get_hash_value(p, elem, image->sha256);
 
-	/* convert the offset handling multiplicative suffixes */
-	image->seek = ustrtoull(seek_str, NULL, 0);
-	if (errno){
-		ERROR("offset argument: ustrtoull failed");
-		return -1;
+	/*
+	 * offset can be set as number or string. As string,
+	 * multiplier suffixes are allowed
+	 */
+	if (offset)
+		image->seek = offset;
+	else {
+		/* convert the offset handling multiplicative suffixes */
+		image->seek = ustrtoull(seek_str, NULL, 0);
+		if (errno){
+			ERROR("offset argument: ustrtoull failed");
+			return -1;
+		}
 	}
 
 	if ((compressed = get_field_string(p, elem, "compressed")) != NULL) {
@@ -925,7 +935,9 @@ static int parser(parsertype p, void *cfg, struct swupdate_cfg *swcfg)
 			return -1;
 		}
 	}
-	get_hw_revision(&swcfg->hw);
+	if (get_hw_revision(&swcfg->hw) < 0) {
+		TRACE("Hardware compatibiliy not found");
+	}
 
 	/* Now parse the single elements */
 	ret = parse_hw_compatibility(p, cfg, swcfg) ||
@@ -974,7 +986,6 @@ int parse_cfg (struct swupdate_cfg *swcfg, const char *filename)
 		fprintf(stderr, "%s:%d - %s\n", config_error_file(&cfg),
 			config_error_line(&cfg), config_error_text(&cfg));
 		config_destroy(&cfg);
-		ERROR(" ..exiting");
 		return -1;
 	}
 
@@ -996,6 +1007,9 @@ int parse_cfg (struct swupdate_cfg __attribute__ ((__unused__)) *swcfg,
 #endif
 
 #ifdef CONFIG_JSON
+
+#define JSON_OBJECT_FREED 1
+
 int parse_json(struct swupdate_cfg *swcfg, const char *filename)
 {
 	int fd, ret;
@@ -1049,7 +1063,9 @@ int parse_json(struct swupdate_cfg *swcfg, const char *filename)
 
 	ret = parser(p, cfg, swcfg);
 
-	json_object_put(cfg);
+	if (json_object_put(cfg) != JSON_OBJECT_FREED) {
+		WARN("Leaking cfg json object");
+	}
 
 	free(string);
 
