@@ -699,12 +699,15 @@ static size_t server_check_during_dwl(char  __attribute__ ((__unused__)) *stream
 	 * a connection parallel to the download
 	 */
 	channel_t *channel = channel_new();
+	if (!channel)
+		return ret;
 
 	if (channel->open(channel, &channel_data_defaults) != CHANNEL_OK) {
 		/*
 		 * it is not possible to check for a cancelUpdate,
 		 * go on downloading
 		 */
+		channel->close(channel);
 		free(channel);
 		return ret;
 	}
@@ -953,7 +956,11 @@ static void *process_notification_thread(void *data)
 	 * used to download the SWU
 	 */
 	channel_t *channel = channel_new();
+	if (!channel)
+		return NULL;
+
 	if (channel->open(channel, &channel_data) != CHANNEL_OK) {
+		channel->close(channel);
 		free(channel);
 		return NULL;
 	}
@@ -1841,7 +1848,7 @@ server_op_res_t server_start(char *fname, int argc, char *argv[])
 			break;
 		case 'n':
 			channel_data_defaults.max_download_speed =
-				(unsigned int)ustrtoull(optarg, 10);
+				(unsigned int)ustrtoull(optarg, NULL, 10);
 			break;
 		/* Ignore not recognized options, they can be already parsed by the caller */
 		case '?':
@@ -1887,6 +1894,8 @@ server_op_res_t server_start(char *fname, int argc, char *argv[])
 		return SERVER_EINIT;
 
 	if (server_hawkbit.channel->open(server_hawkbit.channel, &channel_data_defaults) != CHANNEL_OK) {
+		(void)server_hawkbit.channel->close(server_hawkbit.channel);
+		free(server_hawkbit.channel);
 		return SERVER_EINIT;
 	}
 	/* If an update was performed, report its status to the hawkBit server
@@ -1926,6 +1935,7 @@ server_op_res_t server_start(char *fname, int argc, char *argv[])
 server_op_res_t server_stop(void)
 {
 	(void)server_hawkbit.channel->close(server_hawkbit.channel);
+	free(server_hawkbit.channel);
 	return SERVER_OK;
 }
 
